/*
 *
 *  Copyright (C) 2014 Jürg Müller, CH-5524
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation version 3 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program. If not, see http://www.gnu.org/licenses/ .
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if defined(__WINDOWS__)
  #include <windows.h>
#endif
#if defined(__LINUX__)
  #include <sys/socket.h>
  #include <netinet/in.h>
#endif

#include "KTcpCanClient.h"

const int SendTimeOut = 1000;
const int ReadTimeOut = 1000;

KTcpCanClient::KTcpCanClient()
{
  mConnected = false;
  Trace = false;
}

bool KTcpCanClient::SendBuffer(const char * aBuffer, int aLength, int TimeOut)
{
  if (!aBuffer || !aLength)
    return true;
  
  return SendBuf(aBuffer, aLength, 0, TimeOut) == aLength;
}

bool KTcpCanClient::WriteLine(const char * aBuffer)
{
  int len =  aBuffer ? (int) strlen(aBuffer) : 0;
  
  if (len)
    if (!SendBuffer(aBuffer, len, SendTimeOut))
      return false;
  
  return SendBuffer("\r\n", 2, len ? SendTimeOut : 100);
}

bool KTcpCanClient::ReadLine(char * aString, int aLength)
{
  aString[0] = 0;
   
  int Len = 0;
  char Temp[4];
 
  bool Ok = false;
  
  while (!Ok &&
         ReceiveBuf(Temp, 1, 0, Len ? 100 : ReadTimeOut) == 1)
  {
    if (Temp[0] == 13)
    {
      
    } else
    if (Temp[0] == 10)
    {
      // Blanks am Zeilenende entfernen
      while (Len > 0 && aString[-1] == ' ')
      {
        aString--;
        Len--;
      }
      *aString = 0;
      Ok = true;
    } else
    if (aLength > 1)
    {
      *aString = Temp[0];
      aString++;
      aLength--;
      Len++;
    }
  }
 
  return Ok;
}

void KTcpCanClient::Disconnect()
{
  if (mConnected)
  {
    // unter Windows SD_BOTH == 2
    // unter Linux SHUT_RDWR == 2
    if (mActive)
      ErrorCheck(shutdown(mSocket, 2), "shutdown");
    
    mConnected = false;
  }
  
  Close();
}

bool KTcpCanClient::Connect(int aTimeout)
{
  sockaddr_in addr;
  
  KIpSocket::Open();
  
  if (mActive && !mConnected)
  {
    GetSocketAddr(addr, mRemoteHost, mRemotePort);
    mConnected = ErrorCheck(connect(mSocket, (sockaddr *)(&addr), sizeof(addr)),
                            "connect") == 0;
    
    if (!mConnected && mBlockMode == bmNonBlocking && aTimeout > 0)
      Select(NULL, &mConnected, NULL, aTimeout);
    
    if (!mConnected)
    {
      Close();
    }
  }
  
  return mConnected;
}

// Anfrage vom Internet Explorer:
//
// GET /path HTTP/1.1              << Unterverzeichnis hier einfügen
// Accept: */*
// Accept-Language: de-ch
// Accept-Encoding: gzip, deflate
// User-Agent: Mozilla/4.0 (compatible; MSIE 6.0; ...
// Host: localhost:8080
// Connection: Keep-Alive
// leere Zeile

bool KTcpCanClient::ReadHttpStream(char * aStream, int aStreamLen, const char * aGetString)
{
  bool Ok = false;
  bool HeadOk = true;
  
   ContentLength = 0;
  if (Connect(1000))
  {
    char Get[512];
    char remote[128];
    const char * ParsePtr;
    
    strcpy(Get, "GET ");
    strcat(Get, mRemotePath);
    strcat(Get, " HTTP/1.1\n");
    sprintf(remote, "Host: %s", mRemoteHost);
    Ok = WriteLine(aGetString ? aGetString : Get);
    if (Ok)
      Ok = WriteLine(remote);
    if (Ok)
      Ok = WriteLine(NULL);
    if (Ok)
    {
      Ok = ReadLine(aStream, aStreamLen);
      if (strncmp("HTTP/1.1 200 OK", aStream, 14))
        HeadOk = false;
      if (Trace)
        printf("Tcp line: %s\n", aStream);
    }
    if (Ok)
    {
      // lesen bis zur ersten Leerzeile
      // (Header wird mit einer Leerzeile abgeschlossen)
      do
      {
        Ok = ReadLine(aStream, aStreamLen);
        // "Content-Length: %d"
        if (Ok)
        {
          if (Trace)
            printf("Tcp line: %s\n", aStream);
          ParsePtr = strstr(aStream, "Content-Length");
          if (ParsePtr)
          {
            ParsePtr += strlen("Content-Length");
            while (*ParsePtr && *ParsePtr <= ' ')
              ParsePtr++;
            if (*ParsePtr == ':')
            {
              ParsePtr++;
              ContentLength = (int)strtol(ParsePtr, NULL, 10);
            }
          }
        }
      } while (Ok && strlen(aStream) != 0);
      
      if (0 < ContentLength && ContentLength < aStreamLen)
        Ok = ReceiveBuf(aStream, ContentLength < aStreamLen ? ContentLength : aStreamLen, 0, 100) == ContentLength;
      else
        Ok = ReadLine(aStream, aStreamLen);
    }
    
    Disconnect();
  }
  
  return Ok && HeadOk;
}

bool KTcpCanClient::ReadElster(char * Buffer)
{
  int Read = ReceiveBuf(Buffer, 1, 0, 500); // timeout: 500 ms
  if (Read == 1 && *Buffer != '\n')  // auf LF synchronisieren
  {
    for (int Tot = Read ; Tot < 3*12; Tot++)
    {
      Read = ReceiveBuf(Buffer + Tot, 1, 0, 5);
      if (Read != 1)
      {
        Buffer[Tot] = 0;
        printf("broken %s\n", Buffer);
        return false;
      }
      if (Buffer[Tot] == '\n')
      {
        Buffer[Tot + 1] = 0;
        return true;
      }
    }
    Buffer[3*12] = 0;
    printf("no LF %s\n", Buffer);
  }
  return false;
}

bool KTcpCanClient::SendElster(char * Buffer, int timeout)
{
  while (strlen(Buffer) && Buffer[strlen(Buffer)-1] == ' ')
    Buffer[strlen(Buffer)-1] = 0;

  strcat(Buffer, "\n");
  
  return SendBuffer(Buffer, (int) strlen(Buffer), timeout);
}

