/*
 *
 *  Copyright (C) 2014 Jürg Müller, CH-5524
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation version 3 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program. If not, see http://www.gnu.org/licenses/ .
 */

#if !defined(NCanUtils_H) 

  #define NCanUtils_H

  #define High(A)     (sizeof(A)/sizeof(A[0]) - 1)

  struct KCanFrame
  {
    unsigned Counter;
    int      TimeStampDay;
    int      TimeStampMs;
    unsigned Id;
    int      Len;
    unsigned Flags;
    unsigned char Data[8];
    
    KCanFrame();
    void Init();
    bool EqualData(const KCanFrame & Frame) const;
    bool IsAnswer(const KCanFrame & Frame) const;
    bool Supported() const;
    unsigned Value() const;
    bool Changed(const KCanFrame & SendFrame) const;
    bool GetDataFromStream(const char * str);
  };

  struct KComfortFrame
  {
    unsigned char Data[12];
  
    unsigned GetCheckSum() const;
    unsigned GetSum() const;
    bool CheckSum() const;
    void SetCheckSum();
    bool SetCanFrame(KCanFrame & Frame) const;
    bool GetCanFrame(const KCanFrame & Frame);
    void SetOk(bool Ok = true);
    void SetOkValue(unsigned Value);
    bool EqualData(const KComfortFrame & Frame) const;
    void SetToString(char * Str) const;
    bool SetFromString(const char * Str, bool AppendCheck = true);
    bool IsInitFrame() const;
    void SetInitOk();
  };

  struct KCan232Frame
  {
    // T14611234 4 00010203             1 + 8 + 1 + 2*8 = 26 Byte max.
    // t550      8 aabbccddeeff0a0b
    char msg[32];
  
    KCan232Frame(const char * str = NULL);
    bool SetCanFrame(KCanFrame & Frame) const;
    bool GetCanFrame(const KCanFrame & Frame);
    bool IsOk() const;
    KCan232Frame & operator =(const char * str);
  };

  namespace NCanUtils
  {
    void SleepMs(int aMs);
    void ArtTime(const struct timeval & Time, int & aDays, int & aMs);
    void ArtTime(int & aDays, int & aMs);
    void GetDateFromDays(int aDays, int & aDay, int & aMonth, int & aYear);
    void PrintTime(const KCanFrame & Frame, char * stream, bool csv);
    void PrintTime(int time_days, int time_ms, char * OutStr, bool csv);
    void PrintMs(int days, int ms, char * stream);
    void PrintDateTime(int days, int ms, char * stream);
    void FormatFrame(const KCanFrame & Frame, bool csv, char * OutStr);
    bool IsHexDigit(char aDigit);
    int GetHexDigit(char aDigit);
    unsigned GetHex(const char * & Line);
    bool GetHexByte(const char * & Line, unsigned char & byte);
    bool GetInt(const char * & Line, int & res);   
  }
#if defined(__CAN__)
  extern unsigned gMainThreadId;
  extern unsigned gProcessId;
#endif

#endif

